@@ -13,7 +13,7 @@ use std::ops::{Add, Sub, Mul, AddAssign, SubAssign, MulAssign};
 
 use itertools::{enumerate, zip};
 
-use {Ix, Ixs, Ix0, Ix1, Ix2, Ix3, IxDyn, Dim, Si, IxDynImpl};
+use {Ix, Ixs, Ix0, Ix1, Ix2, Ix3, Ix4, Ix5, Ix6, IxDyn, Dim, Si, IxDynImpl};
 use IntoDimension;
 use RemoveAxis;
 use {ArrayView1, ArrayViewMut1};
@@ -66,6 +66,8 @@ pub trait Dimension : Clone + Eq + Debug + Send + Sync + Default +
     type Pattern: IntoDimension<Dim=Self>;
     /// Next smaller dimension (if applicable)
     type Smaller: Dimension;
+    /// Next larger dimension
+    type Larger: Dimension;
     #[doc(hidden)]
     fn ndim(&self) -> usize;
 
@@ -389,6 +391,9 @@ pub trait Dimension : Clone + Eq + Debug + Send + Sync + Default +
         }
     }
 
+    #[doc(hidden)]
+    fn insert_axis(&self, axis: Axis) -> Self::Larger;
+
     #[doc(hidden)]
     fn try_remove_axis(&self, axis: Axis) -> Self::Smaller;
 
@@ -409,11 +414,23 @@ fn abs_index(len: Ixs, index: Ixs) -> Ix {
 
 // Dimension impls
 
+macro_rules! impl_insert_axis_array(
+    ($n:expr) => (
+        fn insert_axis(&self, axis: Axis) -> Self::Larger {
+            debug_assert!(axis.index() <= $n);
+            let mut out = [1; $n + 1];
+            out[0..axis.index()].copy_from_slice(&self.slice()[0..axis.index()]);
+            out[axis.index()+1..$n+1].copy_from_slice(&self.slice()[axis.index()..$n]);
+            Dim(out)
+        }
+    );
+);
 
 impl Dimension for Dim<[Ix; 0]> {
     type SliceArg = [Si; 0];
     type Pattern = ();
     type Smaller = Self;
+    type Larger = Ix1;
     // empty product is 1 -> size is 1
     #[inline]
     fn ndim(&self) -> usize { 0 }
@@ -430,6 +447,8 @@ impl Dimension for Dim<[Ix; 0]> {
         None
     }
     #[inline]
+    impl_insert_axis_array!(0);
+    #[inline]
     fn try_remove_axis(&self, _ignore: Axis) -> Self::Smaller {
         *self
     }
@@ -442,6 +461,7 @@ impl Dimension for Dim<[Ix; 1]> {
     type SliceArg = [Si; 1];
     type Pattern = Ix;
     type Smaller = Ix0;
+    type Larger = Ix2;
     #[inline]
     fn ndim(&self) -> usize { 1 }
     #[inline]
@@ -517,6 +537,8 @@ impl Dimension for Dim<[Ix; 1]> {
         }
     }
     #[inline]
+    impl_insert_axis_array!(1);
+    #[inline]
     fn try_remove_axis(&self, axis: Axis) -> Self::Smaller {
         self.remove_axis(axis)
     }
@@ -527,6 +549,7 @@ impl Dimension for Dim<[Ix; 2]> {
     type SliceArg = [Si; 2];
     type Pattern = (Ix, Ix);
     type Smaller = Ix1;
+    type Larger = Ix3;
     #[inline]
     fn ndim(&self) -> usize { 2 }
     #[inline]
@@ -644,6 +667,8 @@ impl Dimension for Dim<[Ix; 2]> {
         }
     }
     #[inline]
+    impl_insert_axis_array!(2);
+    #[inline]
     fn try_remove_axis(&self, axis: Axis) -> Self::Smaller {
         self.remove_axis(axis)
     }
@@ -654,6 +679,7 @@ impl Dimension for Dim<[Ix; 3]> {
     type SliceArg = [Si; 3];
     type Pattern = (Ix, Ix, Ix);
     type Smaller = Ix2;
+    type Larger = Ix4;
     #[inline]
     fn ndim(&self) -> usize { 3 }
     #[inline]
@@ -750,6 +776,8 @@ impl Dimension for Dim<[Ix; 3]> {
         order
     }
     #[inline]
+    impl_insert_axis_array!(3);
+    #[inline]
     fn try_remove_axis(&self, axis: Axis) -> Self::Smaller {
         self.remove_axis(axis)
     }
@@ -757,11 +785,12 @@ impl Dimension for Dim<[Ix; 3]> {
 }
 
 macro_rules! large_dim {
-    ($n:expr, $name:ident, $($ix:ident),+) => (
+    ($n:expr, $name:ident, $pattern:ty, $larger:ty, { $($insert_axis:tt)* }) => (
         impl Dimension for Dim<[Ix; $n]> {
             type SliceArg = [Si; $n];
-            type Pattern = ($($ix,)*);
+            type Pattern = $pattern;
             type Smaller = Dim<[Ix; $n - 1]>;
+            type Larger = $larger;
             #[inline]
             fn ndim(&self) -> usize { $n }
             #[inline]
@@ -773,6 +802,8 @@ macro_rules! large_dim {
             #[inline]
             fn slice_mut(&mut self) -> &mut [Ix] { self.ixm() }
             #[inline]
+            $($insert_axis)*
+            #[inline]
             fn try_remove_axis(&self, axis: Axis) -> Self::Smaller {
                 self.remove_axis(axis)
             }
@@ -781,9 +812,22 @@ macro_rules! large_dim {
     )
 }
 
-large_dim!(4, Ix4, Ix, Ix, Ix, Ix);
-large_dim!(5, Ix5, Ix, Ix, Ix, Ix, Ix);
-large_dim!(6, Ix6, Ix, Ix, Ix, Ix, Ix, Ix);
+large_dim!(4, Ix4, (Ix, Ix, Ix, Ix), Ix5, {
+    impl_insert_axis_array!(4);
+});
+large_dim!(5, Ix5, (Ix, Ix, Ix, Ix, Ix), Ix6, {
+    impl_insert_axis_array!(5);
+});
+large_dim!(6, Ix6, (Ix, Ix, Ix, Ix, Ix, Ix), IxDyn, {
+    fn insert_axis(&self, axis: Axis) -> Self::Larger {
+        debug_assert!(axis.index() <= self.ndim());
+        let mut out = Vec::with_capacity(self.ndim() + 1);
+        out.extend_from_slice(&self.slice()[0..axis.index()]);
+        out.push(1);
+        out.extend_from_slice(&self.slice()[axis.index()..self.ndim()]);
+        Dim(out)
+    }
+});
 
 /// IxDyn is a "dynamic" index, pretty hard to use when indexing,
 /// and memory wasteful, but it allows an arbitrary and dynamic number of axes.
@@ -792,6 +836,7 @@ impl Dimension for IxDyn
     type SliceArg = [Si];
     type Pattern = Self;
     type Smaller = Self;
+    type Larger = Self;
     #[inline]
     fn ndim(&self) -> usize { self.ix().len() }
     #[inline]
@@ -808,6 +853,12 @@ impl Dimension for IxDyn
         IxDyn::zeros(self.ndim())
     }
 
+    #[inline]
+    fn insert_axis(&self, axis: Axis) -> Self::Larger {
+        debug_assert!(axis.index() <= self.ndim());
+        Dim::new(self.ix().insert(axis.index()))
+    }
+
     #[inline]
     fn try_remove_axis(&self, axis: Axis) -> Self::Smaller {
         if self.ndim() > 0 {
